#import "FFFastImageViewManager.h"
#import "FFFastImageView.h"

#import <SDWebImage/SDWebImagePrefetcher.h>
#import <SDWebImage/SDImageCache.h>

@implementation FFFastImageViewManager

RCT_EXPORT_MODULE(FastImageView)

- (FFFastImageView*)view {
  return [[FFFastImageView alloc] init];
}

RCT_EXPORT_VIEW_PROPERTY(source, FFFastImageSource)
RCT_EXPORT_VIEW_PROPERTY(resizeMode, RCTResizeMode)
RCT_EXPORT_VIEW_PROPERTY(onFastImageLoadStart, RCTDirectEventBlock)
RCT_EXPORT_VIEW_PROPERTY(onFastImageProgress, RCTDirectEventBlock)
RCT_EXPORT_VIEW_PROPERTY(onFastImageError, RCTDirectEventBlock)
RCT_EXPORT_VIEW_PROPERTY(onFastImageLoad, RCTDirectEventBlock)
RCT_EXPORT_VIEW_PROPERTY(onFastImageLoadEnd, RCTDirectEventBlock)
RCT_REMAP_VIEW_PROPERTY(tintColor, imageColor, UIColor)

RCT_EXPORT_METHOD(preload:(nonnull NSArray<FFFastImageSource *> *)sources resolve:(RCTPromiseResolveBlock)resolve reject:(RCTPromiseRejectBlock)reject)
{
    NSMutableArray *urls = [NSMutableArray arrayWithCapacity:sources.count];

    [sources enumerateObjectsUsingBlock:^(FFFastImageSource * _Nonnull source, NSUInteger idx, BOOL * _Nonnull stop) {
        [source.headers enumerateKeysAndObjectsUsingBlock:^(NSString *key, NSString* header, BOOL *stop) {
            [[SDWebImageDownloader sharedDownloader] setValue:header forHTTPHeaderField:key];
        }];
        [urls setObject:source.url atIndexedSubscript:idx];
    }];

    [[SDWebImagePrefetcher sharedImagePrefetcher] prefetchURLs:urls progress:^(NSUInteger noOfFinishedUrls, NSUInteger noOfTotalUrls) {
        RCTLogInfo(@"preload progress %ld / %ld", noOfFinishedUrls, noOfTotalUrls);
    } completed:^(NSUInteger noOfFinishedUrls, NSUInteger noOfSkippedUrls) {
        RCTLogInfo(@"preload completed %ld - %ld", noOfFinishedUrls, noOfSkippedUrls);
        resolve(@(noOfFinishedUrls));
    }];
}

RCT_EXPORT_METHOD(preloadSingle:(FFFastImageSource *)source resolve:(RCTPromiseResolveBlock)resolve reject:(RCTPromiseRejectBlock)reject)
{
    [[SDWebImagePrefetcher sharedImagePrefetcher] prefetchURLs:@[source.url] progress:^(NSUInteger noOfFinishedUrls, NSUInteger noOfTotalUrls) {
        // RCTLogInfo(@"preload progress %ld / %ld", noOfFinishedUrls, noOfTotalUrls);
    } completed:^(NSUInteger noOfFinishedUrls, NSUInteger noOfSkippedUrls) {
        // RCTLogInfo(@"preload completed %@ %ld - %ld", source.url, noOfFinishedUrls, noOfSkippedUrls);
        NSString* cachePath = [[SDImageCache sharedImageCache] cachePathForKey:source.url.absoluteString];
        [[SDImageCache sharedImageCache] diskImageExistsWithKey:source.url.absoluteString completion:^(BOOL isInCache) {
            if (noOfFinishedUrls == 1 && isInCache) {
                // RCTLogInfo(@"---> completionBlock preloadSingle %@ %@ %@", source.url, cachePath, isInCache ? @"YES" : @"NO");
                resolve(cachePath);
            } else {
                reject(@"errr", @"unknown error", nil);
            }
        }];
    }];
}

RCT_EXPORT_METHOD(putSource:(NSString *)url body:(NSString *)body resolve:(RCTPromiseResolveBlock)resolve reject:(RCTPromiseRejectBlock)reject)
{
    NSString* dest = [[SDImageCache sharedImageCache] cachePathForKey:url];
    NSData *data = [[NSData alloc] initWithBase64EncodedString:body options:NSDataBase64DecodingIgnoreUnknownCharacters];
    [[SDImageCache sharedImageCache] storeImageDataToDisk:data forKey:url];
    resolve(dest);
}

RCT_EXPORT_METHOD(putFile:(NSString *)url path:(NSString *)path resolve:(RCTPromiseResolveBlock)resolve reject:(RCTPromiseRejectBlock)reject)
{
    NSString* dest = [[SDImageCache sharedImageCache] cachePathForKey:url];
    NSError* error;
    
    NSString* src;
    
    if ([path hasPrefix:@"file://"]) {
        src = [[NSURL URLWithString:path] path];
    } else {
        src = path;
    }
    
    [[NSFileManager defaultManager] copyItemAtPath:src toPath:dest error:&error];
    
    if (error) {
        reject(@"error", @"unknown error", error);
    } else {
        resolve(dest);
    }
}

RCT_EXPORT_METHOD(cacheExist:(FFFastImageSource *)source resolve:(RCTPromiseResolveBlock)resolve reject:(RCTPromiseRejectBlock)reject)
{
    NSString* cachePath = [[SDImageCache sharedImageCache] cachePathForKey:source.url.absoluteString];
    [[SDImageCache sharedImageCache] diskImageExistsWithKey:source.url.absoluteString completion:^(BOOL isInCache) {
        RCTLogInfo(@"preload completed diskImageExistsWithKey %@ %@", source.url, isInCache ? @"YES" : @"NO");
        if (isInCache) {
            resolve(cachePath);
        } else {
            resolve(nil);
        }
    }];
}

@end

